unit fcl_misc;

{ unit fcl_misc.pp: miscalaneous procedures and functions for Fpc

  Copyright (C) 2011 by Bart Broersma

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
}


{$IFDEF FPC}
 {$mode objfpc}
{$ENDIF}
{$H+}

Interface

uses
  SysUtils, Classes, ExtMasks;


function BoolStr(const Value: Boolean): String;
function LocalizeBuildDateString(BuildDate: String): String; overload;
function LocalizeBuildDateString(BuildDate: String; ADateFormat: String): String; overload;

function HtmlEscape(S: String): String;

const
   {$IFDEF UNIX}
   FlyingSheep = 'FlyingSheep';
   IniExt = '.conf';
   {$ELSE}
   FlyingSheep = 'Flying Sheep Inc';
   IniExt = '.ini';
   {$ENDIF}

function GetAppConfigRoot(const Global: Boolean): String;
function GetFlyingSheepRoot(const Global: Boolean): String;
function GetFlyingSheepConfigDir(const Global: Boolean; const AppName: String): STring;

function GetExtensionFromFilterAtIndex(Filter: String; Index: Integer): String;
function GetFilterIndexFromFileName(Filter, FileName: String; const CaseSensitive: Boolean = False): Integer;

Implementation


function BoolStr(const Value: Boolean): String;
begin
  if Value then Result := 'True' else Result := 'False';
end;



function LocalizeBuildDateString(BuildDate: String): String;
begin
  Result := LocalizeBuildDateString(BuildDate,'');
end;

function LocalizeBuildDateString(BuildDate: String; ADateFormat: String): String;
//use {$I %DATE% as input!} (it MUST be in y/m/d/ format!)
function StrToWord(const S: String): Word;
var
  L: LongInt;
begin
  if TryStrToInt(S, L) then
  begin
    if (L >=0) and (L < 65536) then Result := Word(L)
    else Result := $FFFF;
  end
  else Result := $FFFF;
end;

var
  SlashPos1, SlashPos2: integer;
  Date: TDateTime;
begin
  Result := 'Unknow build date';
  //Input is in y/m/d format
  SlashPos1 := Pos('/',BuildDate);
  SlashPos2 := SlashPos1 +
    Pos('/', Copy(BuildDate, SlashPos1+1, Length(BuildDate)-SlashPos1));
  if (SlashPos1 = 0) or (SlashPos2 = 0) then exit;
  if not TryEncodeDate(StrToWord(Copy(BuildDate,1,SlashPos1-1)),
    StrToWord(Copy(BuildDate,SlashPos1+1,SlashPos2-SlashPos1-1)),
    StrToWord(Copy(BuildDate,SlashPos2+1,Length(BuildDate)-SlashPos2)),Date) then exit;
  if (ADateFormat = '') then ADateFormat := 'dd-mm-yyyy';
  Result := FormatDateTime(ADateFormat, Date);
end;

function HtmlEscape(S: String): String;
begin
  S := StringReplace(S, '<','&lt',[rfReplaceAll]);
  S := StringReplace(S, '>','&gt',[rfReplaceAll]);
  S := StringReplace(S, '&','&amp',[rfReplaceAll]);
  Result := S;
end;


//Dummy functions for GetAppConfigRoot: they cannot be local functions
function GetAppName: String;
begin
  Result := '';
end;
function GetVendor: String;
begin
  Result := '';
end;

function GetAppConfigRoot(const Global: Boolean): String;
var
  OldGetAppName: TGetAppNameEvent;
  OldGetVendor: TGetVendorNameEvent;
begin
  OldGetAppName := OnGetApplicationName;
  OnGetApplicationName := @GetAppName;
  OldGetVendor := OnGetVendorName;
  OnGetVendorName := @GetVendor;
  try
    Result := GetAppConfigDir(Global);
  finally
    OnGetApplicationName := OldGetAppName;
    OnGetVendorName := OldGetVendor;
  end;
end;

function GetFlyingSheepRoot(const Global: Boolean): String;
var
  Root: String;
begin
  Root := GetAppConfigRoot(Global);
  Root := IncludeTrailingPathDelimiter(Root);
  Result := Root + FlyingSheep;
end;

function GetFlyingSheepConfigDir(const Global: Boolean; const AppName: String): STring;
begin
  Result := IncludeTrailingPathDelimiter(GetFlyingSheepRoot(Global)) + AppName;
end;


function GetExtensionFromFilterAtIndex(Filter: String; Index: Integer): String;
{
  Returns a file extension from a filter as used in TOpen/TSaveDialog
  - it will return the extension (including the leading period) that matches the index (index starts at 1)
  - it will return an empty string if the extension contains a wildcard, or on any failure
  - filters have the format of:
    'Text files (*.txt)|*.txt|'+
    'Pascal files (*.pp;*.pas)|*.pp;*.pas|'+
    'All files (*.*)|*.*'
  - if a given extension is a composite (like '*.pp;*.pas') it will return the first one from the list
}
var
  p, pipe: Integer;
begin
  Result := '';
  if Index < 1 then Exit;
  p := 0;
  pipe := 0;
  //Find where the filter for the given index starts
  while (p < Length(Filter)) do
  begin
    Inc(p);
    //Debugln('p = ',dbgs(p),' Filter[',dbgs(p),'] = ',Filter[p]);
    if Filter[p] = '|' then Inc(pipe);
    if (pipe = 2 * (Index - 1)) then break;
  end;
  //debugln('p = ',dbgs(p),' pipe = ',dbgs(pipe));
  if (p = length(Filter)) then exit;
  System.Delete(Filter,1,p);
  //Find the | that splits the filter name and the filter extension
  p := Pos('|',Filter);
  if (p = 0) then exit;
  System.Delete(Filter,1,p);
  Filter := Copy(Filter,1,MaxInt);
  //debugln('Filter now = ',filter);

  //The associated extension ends at the first ; or |, or at the end of the string
  p := Pos(';',Filter);
  pipe := Pos('|',Filter);
  //Debugln('Pos(;/|,Filter) = ',dbgs(p),' ',dbgs(pipe));
  if (pipe < p) or (p = 0) then p := pipe;
  if (p > 0) then System.Delete(Filter,p,Length(Filter) - p + 1);
  //debugln('Filter now = ',filter);

  //Get the associated extension
  Filter := ExtractFileExt(Filter);
  //debugln('Filter now = ',filter);
  //if an extension at this point contains a wildcard, reject it
  if (Pos('?',Filter) > 0) or (Pos('*',Filter) > 0) then exit;
  Result := Filter;
end;

function GetFilterIndexFromFileName(Filter, FileName: String; const CaseSensitive: Boolean = False): Integer;
var
  i,p: Integer;
  InFilter: Boolean;
  OnlyExtensions: String;
  SL: TStringList;
begin
  Result := 0;
  p := 0;
  InFilter := False;
  OnlyExtensions := '';
  while (p < length(Filter)) do
  begin
    inc(p);
    if Filter[p] = '|' then
    begin
      if (not InFilter) and (OnlyExtensions <> '') then OnlyExtensions := OnlyExtensions + LineEnding;
      InFilter := not InFilter;
    end
    else
    if InFilter then OnlyExtensions := OnlyExtensions + Filter[p];
  end;
  OnlyExtensions := Trim(OnlyExtensions);
  if (OnlyExtensions = '') then exit;
  SL := TStringList.Create;
  try
    SL.Text := OnlyExtensions;
    for i := 0 to SL.Count - 1 do
    begin
      if MatchesMaskList(FileName, SL.Strings[i], CaseSensitive) then
      begin
        Result := i + 1;
        Break;
      end;
    end;
  finally
    SL.Free;
  end;
end;

end.
